using FitConnect;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.Logging;

namespace ConsoleAppExample;

public static class SubscriberDemo {
    public static async Task Run(IConfiguration config, ILogger logger) {
        var clientId = config["FitConnect:Subscriber:ClientId"];
        var clientSecret = config["FitConnect:Subscriber:ClientSecret"];
        var privateKeyDecryption = config["FitConnect:Subscriber:PrivateKeyDecryption"];
        var privateKeySigning = config["FitConnect:Subscriber:PrivateKeySigning"];


        OutputHelper.PrintSubscriber();

        var subscriber = Client.GetSubscriber(FitConnectEnvironment.Testing, clientId,
            clientSecret,
            await File.ReadAllTextAsync(privateKeyDecryption),
            await File.ReadAllTextAsync(privateKeySigning),
            logger);

        var submissions = await subscriber.GetAvailableSubmissionsAsync();

        foreach (var submission in submissions) {
            var outputDirectory = Directory.CreateDirectory(
                Path.Combine(".", "submissions", submission.Id, "attachments"));
            try {
                outputDirectory.Create();
                var requestedSubmission =
                    await subscriber.RequestSubmissionAsync(submission.Id);
                var attachments = await requestedSubmission.GetAttachmentsAsync();
                foreach (var attachment in attachments)
                    await File.WriteAllBytesAsync(
                        outputDirectory.FullName + "/" + (attachment.Filename ?? "temp.dat"),
                        attachment.Content);

                logger.LogInformation("Fachdaten: {Data}", requestedSubmission.GetDataString());
                await requestedSubmission
                    .AcceptSubmissionAsync();
            }
            catch (Exception e) {
                logger.LogError(e, "Fehler beim Abrufen der Einreichung");
            }
        }
    }
}
