#nullable disable
using Newtonsoft.Json;

namespace FitConnect.Models.v1.Api;

public partial class Problems {
    public enum ProblemInstanceEnum {
        Submission,
        Metadata,
        Data
    }

    public enum ProblemTypeEnum {
        AttachmentMismatch,
        MissingAttachments,
        EncryptionIssue,
        HashMismatch,
        IncorrectAuthenticationTag,
        InvalidEventLog,
        MissingAuthenticationTag,
        MissingData,
        MissingSchema,
        ServiceMismatch,
        SchemaViolation,
        SyntaxViolation,
        UnsupportedDataSchema,
        UnsupportedMetaSchema,
        UnsupportedService,
        UnsupportedReplyChannel
    }

    internal Problems() {
    }

    public Problems(ProblemTypeEnum problemType, string detail) {
        type = "https://schema.fitko.de/fit-connect/events/problems/";
        switch (problemType) {
            case ProblemTypeEnum.MissingAttachments:
                type += "missing-attachment";
                title = TitleAttachmentsMismatch;
                instance = "submission";
                break;
            case ProblemTypeEnum.AttachmentMismatch:
                type += "attachments-mismatch";
                title = TitleEventLogInconsistent;
                instance = "submission";
                break;
            case ProblemTypeEnum.EncryptionIssue:
                type += "encryption-issue";
                title = TitleDecryptionIssue;
                instance = "metadata";
                break;
            case ProblemTypeEnum.HashMismatch:
                type += "hash-mismatch";
                title = TitleEventLogInconsistent;
                instance = "submission";
                break;
            case ProblemTypeEnum.IncorrectAuthenticationTag:
                type += "incorrect-authentication-tag";
                title = TitleEventLogInconsistent;
                instance = "submission";
                break;
            case ProblemTypeEnum.InvalidEventLog:
                type += "invalid-event-log";
                title = TitleEventLogInconsistent;
                instance = "submission";
                break;
            case ProblemTypeEnum.MissingAuthenticationTag:
                type += "missing-authentication-tag";
                title = TitleAuthenticationTagMissing;
                instance = "submission";
                break;
            case ProblemTypeEnum.MissingData:
                type += "missing-data";
                title = TitleMissingData;
                instance = "metadata";
                break;
            case ProblemTypeEnum.MissingSchema:
                type += "missing-schema";
                title = TitleMissingSchema;
                instance = "metadata";
                break;
            case ProblemTypeEnum.ServiceMismatch:
                type += "service-mismatch";
                title = TitleServiceMismatch;
                instance = "metadata";
                break;
            case ProblemTypeEnum.SchemaViolation:
                type += "schema-violation";
                title = TitleSchemaViolationData;
                instance = "data";
                break;
            case ProblemTypeEnum.SyntaxViolation:
                type += "syntax-violation";
                title = TitleSyntaxViolation;
                instance = "metadata";
                break;
            case ProblemTypeEnum.UnsupportedDataSchema:
                type += "unsupported-schema";
                title = TitleUnsupportedDataSchema;
                instance = "metadata";
                break;
            case ProblemTypeEnum.UnsupportedMetaSchema:
                type += "unsupported-schema";
                title = TitleUnsupportedSchema;
                instance = "metadata";
                break;
            case ProblemTypeEnum.UnsupportedService:
                type += "unsupported-service";
                title = TitleUnsupportedService;
                instance = "metadata";
                break;
            case ProblemTypeEnum.UnsupportedReplyChannel:
                type += "unsupported-reply-channel";
                title = TitleUnsupportedReplyChannel;
                instance = "metadata";
                break;
            default:
                throw new ArgumentOutOfRangeException(nameof(problemType), problemType, null);
        }

        this.detail = detail;
    }


    public Problems(ProblemTypeEnum problemType, string title, string detail,
        string problemInstance) : this(problemType, detail) {
        this.title = title ?? problemType switch {
            ProblemTypeEnum.InvalidEventLog => TitleEventLogInconsistent,
            _ => problemType.ToString()
        };

        instance = problemInstance;
    }

    public Problems(ProblemTypeEnum problemType, string title, string detail,
        ProblemInstanceEnum problemInstance) : this(problemType, detail) {
        this.title = title ?? problemType switch {
            ProblemTypeEnum.InvalidEventLog => TitleEventLogInconsistent,
            _ => problemType.ToString()
        };

        instance = problemInstance switch {
            ProblemInstanceEnum.Metadata => "metadata",
            ProblemInstanceEnum.Submission => "submission",
            ProblemInstanceEnum.Data => "data",
            _ => throw new ArgumentOutOfRangeException(nameof(problemInstance), problemType, null)
        };
    }


    [JsonProperty("instance", NullValueHandling = NullValueHandling.Ignore)]
    public string instance { get; set; }

    [JsonProperty("detail", NullValueHandling = NullValueHandling.Ignore)]
    public string detail { get; set; }

    [JsonProperty("title", NullValueHandling = NullValueHandling.Ignore)]
    public string title { get; set; }

    // [JsonProperty("description", NullValueHandling = NullValueHandling.Ignore)]
    // public string? description { get; set; }

    [JsonProperty("type", NullValueHandling = NullValueHandling.Ignore)]
    public string type { get; set; }

    // [JsonProperty("items", NullValueHandling = NullValueHandling.Ignore)]
    // public Items? items { get; set; }
}
